// ahrs.h

#include "math.h"

typedef double	f_t;	// base type of variable for calcs
typedef f_t		v_t3[3]; // vector length 3
typedef f_t		v_t[7]; // vector length 7
typedef v_t		m_t[7]; // matrix dims 7x7
typedef int		iter_t; // iteration variables


//vector functions

void v_zero(v_t c, iter_t n);

 void v_add(
	v_t 		c,
	const v_t 	a,
	const v_t 	b,
	iter_t		n
);

 void v_sub(
	v_t 		c,
	const v_t 	a,
	const v_t 	b,
	iter_t		n
);

 void v_scale(
	v_t			c,
	const v_t	a,
	f_t			s,
	iter_t		n
);
 
//matrix functions

 void m_zero(
	m_t			c,
	iter_t		m,
	iter_t		n
);
 
 void m_add(
	m_t 		c,
	m_t 		a,
	m_t			b,
	iter_t		m,
	iter_t		n
);

 void m_sub(
	m_t 		c,
	m_t 		a,
	m_t			b,
	iter_t		m,
	iter_t		n
);
 
 void m_scale(
	m_t			c,
	m_t			a,
	f_t			s,
	iter_t		m,
	iter_t		n
);
 
 void mv_mult(
	v_t			vc,
	m_t			ma,
	const v_t	vb,
	iter_t		m,
	iter_t		n
);
 
 void vm_mult(
	v_t			vc,
	v_t			va,
	m_t			mb,
	iter_t		m,
	iter_t		n
);
 
 void m_mult(
	m_t			c,
	m_t			a,
	m_t			b,
	iter_t		m,
	iter_t		n,
	iter_t		p
);

 void m_transpose(
	m_t			c,
	m_t			a,
	iter_t		m,
	iter_t		n
);
 
 f_t m33_det(m_t m);

 void m33_inv(
	m_t			D,
	m_t			m
);

 void m_print(
	const char *label,
	m_t			a,
	iter_t		m,
	iter_t		n
);

void v_print(
	const char *label,
	v_t			a,
	iter_t		n
);
 
void compute_f(
	v_t			F,
	m_t 		DCM
);

void compute_c(
	m_t			C,
	v_t			X
);

// kalman stuff
 
void kalman_update(
	//m_t			P,
	//m_t			R,
	//m_t			C,
	//v_t			X,
	//const v_t	Xe,
	//const v_t	Xm
	v_t	Xe,
	v_t	Xm
);

void ahrs_init(
	f_t			ax,
	f_t			ay,
	f_t			az,
	f_t			b_p,
	f_t			b_q,
	f_t			b_r,
	f_t			heading
);

void ahrs_step(
	v_t			angles_out,
	f_t			dt,
	f_t			p,
	f_t			q,
	f_t			r,
	f_t			ax,
	f_t			ay,
	f_t			az,
	f_t			heading
);

// correct angles to ECEF reference frame based on gps acceleration
void body_to_ecef(	v_t e_angles,
					f_t			ax, 	// measured accels
					f_t			ay,
					f_t			az,
					f_t 		agpsx,
					f_t 		agpsy,
					f_t 		agpsz
				);	

//Vector helpers related to the AHRS computation

f_t sq(f_t d);

void v_normq(
	v_t			q_out,
	const v_t	q
);
 
void quat2euler(
	v_t			euler,
	const v_t	q
);
 
void euler2quat(
	v_t			q,
	f_t			phi,
	f_t			theta,
	f_t			psi
);

void accel2euler(
	v_t			euler,
	f_t			ax,
	f_t			ay,
	f_t			az,
	f_t			compass
);
 
void quat2dcm(
	m_t			dcm,
	const v_t	q
);
 
void build_A(
	m_t			W,
	f_t			p,
	f_t			q,
	f_t			r
);

void build_C(m_t W, v_t q);
