/*
 *
 * microcontroller.c
 * note - uses internal FRC and EMUC2/EMUD2 
 * Mitch
 *
 */	

#include "micro_frc.h"

RCONBITS RCONbits_backup;


// Internal FRC Oscillator
_FOSCSEL(FNOSC_FRCPLL); 								// FRC Oscillator
_FOSC(FCKSM_CSECMD & OSCIOFNC_ON & POSCMD_NONE);	// Clock Switching is enabled and Fail Safe Clock Monitor is disabled
													// OSC2 Pin Function: OSC2 is Clock Output
													// Primary Oscillator Mode Disabled
_FWDT(FWDTEN_OFF); 									// Watchdog Timer Enabled/disabled by user software
													// (LPRC can be disabled by clearing SWDTEN bit in RCON register	
//_FICD(ICS_PGD1);
_FICD(ICS_PGD2);
//_FICD(ICS_PGD3);




void microcontroller_init()
{	
	// Disable Watch Dog Timer
	RCONbits.SWDTEN=0;
	
// Configure Oscillator to operate the device at 40MIPS
// Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
// Fosc= 7.37M*43/(2*2)=79.22Mhz for ~40MIPS input clock

PLLFBD=41; // M=43
CLKDIVbits.PLLPOST=0; // N1=2
CLKDIVbits.PLLPRE=0; // N2=2
OSCTUN=0; // Tune FRC oscillator, if FRC is used


// Clock switch to incorporate PLL
__builtin_write_OSCCONH(0x01); 		// Initiate Clock Switch to
									// FRC with PLL (NOSC=0b001)
__builtin_write_OSCCONL(0x01); 		// Start clock switching
while (OSCCONbits.COSC != 0b001); 	// Wait for Clock switch to occur

// Wait for PLL to lock
while(OSCCONbits.LOCK!=1) {};
	
	RCONbits_backup = RCONbits;
	RCON = 0;  // we have to do this to make sure we can read RCON correctly on a (next) reset
}


/*!
 *  Delay function
 *  @param ms Number of milliseconds (0.001s) the function needs to wait.
 */
void microcontroller_delay_ms(unsigned long ms)
{
	//__delay32((long)FCY/1000L*ms);
	unsigned long i = FCY/1000/12*ms;
    for(;i > 0 ;i--)
        asm("clrwdt");
}


/*!
 *  Delay function
 *  @param us Number of microseconds (0.000001s) the function needs to wait.
 */
void microcontroller_delay_us(unsigned long us)
{
	long i = FCY/1000/1000/12*us;
    for(;i > 0 ;i--)
        asm("clrwdt");
}


int microcontroller_after_reboot()
{
	return !(RCONbits_backup.EXTR || RCONbits_backup.POR); // We are in a reboot if there was no external reset
}


/* ****************************************************************
* Standard Exception Vector handlers if ALTIVT (INTCON2<15>) = 0  *
*                                                                 *
* Not required for labs but good to always include                *
******************************************************************/
#define _trapISR __attribute__((interrupt,no_auto_psv))

void _trapISR _OscillatorFail(void)
{

        INTCON1bits.OSCFAIL = 0;
        //printf("Oscillator error!");
        //while(1);
}

void _trapISR _AddressError(void)
{
        INTCON1bits.ADDRERR = 0;
        //printf("Address error!");
        //while(1);
        asm("reset");
}

void _trapISR _StackError(void)
{
        INTCON1bits.STKERR = 0;
        //printf("Stack error!");
        //while(1);
}

void _trapISR _MathError(void)
{
        INTCON1bits.MATHERR = 0;
        //printf("Math error!");
        //while(1);
}
