// spi.c

# include "sensor_spi.h"

#define roll  	0
#define pitch 	1
#define yaw		2
#define accel_z 3
#define accel_y 4	// this is the x axis from the lis chip
#define accel_x 5	// this is the y axis 

#define CS0 PORTBbits.RB0 // chip select outputs
#define CS1 PORTBbits.RB1
#define CS2 PORTBbits.RB2
#define CS3 PORTBbits.RB3
#define CS4 PORTBbits.RB4


typedef struct tagbits {
        unsigned b0    :1;
        unsigned b1    :1;
        unsigned b2    :1;
        unsigned b3    :1;
        unsigned b4    :1;
        unsigned b5    :1;
        unsigned b6    :1;
        unsigned b7    :1;
} BITS;

typedef struct tagbytes {
        char byte0;
        char byte1;
} BYTES;

typedef union temp 
	{	char a_byte;
		BITS b_8;
	} byte_union;

typedef	union temp1 
	{
  		int i16;
  		BYTES i8;
		struct {char dummy;
    			unsigned b0    :1;
        		unsigned b1    :1;
        		unsigned b2    :1;
        		unsigned b3    :1;
        		unsigned b4    :1;
        		unsigned b5    :1;
        		unsigned b6    :1;
        		unsigned b7    :1;
				} b8;
	} word_union;




//volatile unsigned int* ADC16Ptr = &ADCBUF0; //Pointer to ADC register buffer, 


// buffer of previous sensor values
int filter_buffer[4][6] = { {0,0,0,0,0,0},{0,0,0,0,0,0},{0,0,0,0,0,0},{0,0,0,0,0,0} };

// array of filtered sensor values
int sensor_buf[6] ={0,0,0,0,0,0};
int current_buf = 1;










// functions to return accelerometer values - are all used as ratios so not scaled
// this axis should be towards the nose of the plane a_x = (a_x + 50) / 1120
float accelero_x()
{	return  ((float)((long)(sensor_buf[accel_x]+50))) 	/114.	;}

// this axis should be towards the right wingtip a_y = -(a_y + 70) / 1100
float accelero_y()
{	return -((float)((long)(sensor_buf[accel_y]+70)))	/112.	;}

// this axis should be towards the ground a_z = a_z / 1100
float accelero_z()
{	return -((float)((long)sensor_buf[accel_z]))	/112.		;}


int ax_raw(void)
{ 	return filter_buffer[current_buf][accel_x];}	// current raw value

int ax_filtered(void)
{ 	return sensor_buf[accel_x];}  // current filtered value

int ay_raw(void)
{ 	return filter_buffer[current_buf][accel_y];}	// current raw value

int ay_filtered(void)
{ 	return sensor_buf[accel_y];}  // current filtered value

int az_raw(void)
{ 	return filter_buffer[current_buf][accel_z];}	// current raw value

int az_filtered(void)
{ 	return sensor_buf[accel_z];}  // current filtered value


// gyros



float gyro_roll_dps() // p = -(p - 1024) * 0.07234			// calibration in deg /sec
{	return -(((float)sensor_buf[roll] -1024)* 0.07234);	}

float gyro_roll_rps()										// rad/sec
{	return deg_to_rad(gyro_roll_dps());	}


float gyro_pitch_dps() // q = (q - 1024) * 0.07234
{	return (((float)sensor_buf[pitch] -1024)* 0.07234);	}

float gyro_pitch_rps()
{	return deg_to_rad(gyro_pitch_dps());	}

float gyro_yaw_dps() //yaw_rate = (yaw_rate - 1024) * 0.07234 '75/1024
{	return (((float)sensor_buf[yaw] -1024)* 0.07234);	}

float gyro_yaw_rps()
{	return deg_to_rad(gyro_yaw_dps());	}








int p_raw(void)
{ 	return filter_buffer[current_buf][roll];}	// current raw value

int p_filtered(void)
{ 	return sensor_buf[roll];}  // current filtered value

int q_raw(void)
{ 	return filter_buffer[current_buf][pitch];}	// current raw value

int q_filtered(void)
{ 	return sensor_buf[pitch];}  // current filtered value

int yaw_raw(void)
{ 	return filter_buffer[current_buf][yaw];}	// current raw value

int yaw_filtered(void)
{ 	return sensor_buf[yaw];}  // current filtered value

float rad_to_deg(float inrad)
{ return (inrad*57.2958); }

float deg_to_rad(float indeg)
{ return (indeg*.01745); }


// update sensor values
void update_sensors(void)
{
	unsigned int i;
	
	// here we get all of the raw sensor values and save to buffer

		start_mlx1();	// start conversions on the gyros - need 250us to settle
		start_mlx2();
		start_mlx3();
  		
		filter_buffer[current_buf][accel_z] = read_lis_z();	//3
		filter_buffer[current_buf][accel_y] = read_lis_y();	//4
		filter_buffer[current_buf][accel_x] = read_lis_x();	//5

		filter_buffer[current_buf][roll] = read_mlx3(); 		//0
		filter_buffer[current_buf][pitch] = read_mlx2();		//1
		filter_buffer[current_buf][yaw] = read_mlx1();		//2

    for (i=0; i < 6; i++) // for each sensor update the filtered value using 4th order Runge-Kutta filter
     	sensor_buf[i] = (filter_buffer[current_buf][i]/6 + 
     	              filter_buffer[(current_buf+1)%4][i]/6 +  // % is modulo to pick correct address
     	              filter_buffer[(current_buf+2)%4][i]/3 +
     	              filter_buffer[(current_buf+3)%4][i]/3);

	current_buf = (current_buf+1) % 4; //increment the buffer	
}

// initialize spi module - uses dsPIC30F/dsPIC33F/PIC24H SPI Peripheral Module Library 
void init_spi(void)
{
	// setup port pins 

	QEI1CON=0;				// disable qei to get pins as digital outputs
	
	AD1PCFGLbits.PCFG0=1; 	//analog off
	AD1PCFGLbits.PCFG1=1;
	AD1PCFGLbits.PCFG2=1;
	AD1PCFGLbits.PCFG3=1;
	AD1PCFGLbits.PCFG4=1;

	TRISBbits.TRISB0=0; 	// all outputs 
	TRISBbits.TRISB1=0;
	TRISBbits.TRISB2=0;
	TRISBbits.TRISB3=0;
	TRISBbits.TRISB4=0;

	PORTBbits.RB0 =	1;		// chip selects hi - they are active low
	PORTBbits.RB1 =	1;
	PORTBbits.RB2 =	1;
	PORTBbits.RB3 =	1;
	PORTBbits.RB4 =	1;


	TRISGbits.TRISG6 = 0;	// sck2 output
	TRISGbits.TRISG7 = 1;	// miso2 sdi2 input
	TRISGbits.TRISG8 = 0;	// mosi2 sdo2 output
	//TRISGbits.TRISG9 = 0;	// was ss2 output
	//TRISEbits.TRISE8 = 1;	// drdy  input (dataready)


	PWM1CON1bits.PEN3L=0;	// disable as pwm pins
	PWM1CON1bits.PEN2H=0;
	TRISEbits.TRISE3 = 0;	// led output
	TRISEbits.TRISE4 = 0;	// led output
	PORTEbits.RE3 = 0;		// green on
	PORTEbits.RE4 = 1;		// red off
     
    //PLLFBD = 38; 
	//CLKDIV = 0x0040;            
	SPI2CON1 = 0b0000000001100000; 	// 0000 0001 0011 1011
									// b14 0 = Framed SPI support disabled
									// b10 0 = 8 bit data transfer
									// !!!!!!!!!!!!!!  b8 1 = Serial output data changes on transition from active clock state to Idle clock state (see bit 6)
									// b6 1 Idle state for clock is a high level; active state is a low level
									// b5 1 = Master mode
									// b4-2 Secondary prescaler 8:1 000
									// b1-0 Primary prescaler 64:1 00
									// 40 mhz fcy 000 01 312 khz 
									

  	SPI2CON2 = 0x0000;		// not framed
 	SPI2STAT = 0x8000;        // enable spi

/*
OpenSPI2(ENABLE_SCK_PIN & ENABLE_SDO_PIN & SPI_MODE16_OFF & SPI_SMP_OFF & SPI_CKE_OFF &
         SLAVE_ENABLE_OFF & MASTER_ENABLE_ON & PRI_PRESCAL_16_1 & SEC_PRESCAL_8_1,
         FRAME_ENABLE_OFF,
         SPI_ENABLE & SPI_RX_OVFLOW_CLR); 
*/


	
}

// SPI2 interrupt not used yet
void __attribute__((__interrupt__,__no_auto_psv__)) _SPI2Interrupt(void) 

{   
	
    IFS2bits.SPI2IF = 0;	//clear interrupt flag

}  


void start_mlx1(void)			// start mlx conversion - should wait 250usec before reading
{		CS1=0;   		// Select MLX1
        WriteSPI (0b10010100); 	// sendRead rate ADCC instruction
		CS1=1;		// Unselext MLX1
		ReadSPI(); // dummy read to clear
} // end start_mlx1


void start_mlx2(void)			// start mlx conversion - should wait 250usec before reading
{		CS2=0;   		// Select MLX1
        WriteSPI (0b10010100); 	// sendRead rate ADCC instruction
		CS2=1;		// Unselext MLX1
		ReadSPI(); // dummy read to clear
} // end start_mlx2


void start_mlx3(void)			// start mlx conversion - should wait 250usec before reading
{		CS3=0;   		// Select MLX1
        WriteSPI (0b10010100); 	// sendRead rate ADCC instruction
		CS3=1;		// Unselext MLX1
		ReadSPI(); // dummy read to clear
} // end start_mlx3


int read_mlx1(void)
{
                 
// define data structure for accessing bits and bytes
	 word_union indata;

#define MSbit indata.b8.b7
#define EOCbit indata.b8.b5
#define hibyte indata.i8.byte1
#define lowbyte indata.i8.byte0
#define	inword indata.i16

	 
    	CS1=0;   		//Select MLX
        WriteSPI (0b10000000); 	// Send SPI ADCR Instruction  
		ReadSPI(); // dummy read to clear
		WriteSPI (0); 	// clock in 8 bits
		hibyte = ReadSPI();		// keep spi module in 8 bit mode
		WriteSPI (0); 	// clock in 8 bits
		lowbyte = ReadSPI();
        CS1=1;			//Unselext MLX

		//rs232_send('1');
        
        if (MSbit == 0 && EOCbit == 1)
        {
            inword >>= 1;		// shift right 1 bit
            inword &= 0b0000011111111111; // mask out only 11 lsbits
			
            return(inword); 
        }
		else return(0); // send zeros for bad data

}		// end read_mlx1






int read_mlx2(void)
{
                 
// define data structure for accessing bits and bytes
	word_union indata;

    	CS2=0;   		//Select MLX
        WriteSPI (0b10000000); 	// Send SPI ADCR Instruction  
		ReadSPI(); // dummy read to clear
		WriteSPI (0); 	// clock in 8 bits
		hibyte = ReadSPI();		// keep spi module in 8 bit mode
		WriteSPI (0); 	// clock in 8 bits
		lowbyte = ReadSPI();
        CS2=1;			//Unselext MLX

		//rs232_send('2');
        
        if (MSbit == 0 && EOCbit == 1)
        {
            inword >>= 1;		// shift right 1 bit
            inword &= 0b0000011111111111; // mask out only 11 lsbits
			
            return(inword); 
        }
		else return(0); // send zeros for bad data

}		// end read_mlx2




int read_mlx3(void)
{
                 
// define data structure for accessing bits and bytes

		word_union indata;

    	CS3=0;   		//Select MLX
        WriteSPI (0b10000000); 	// Send SPI ADCR Instruction  
		ReadSPI(); // dummy read to clear
		WriteSPI (0); 	// clock in 8 bits
		hibyte = ReadSPI();		// keep spi module in 8 bit mode
		WriteSPI (0); 	// clock in 8 bits
		lowbyte = ReadSPI();
        CS3=1;			//Unselext MLX
        
		//rs232_send('3');

        if (MSbit == 0 && EOCbit == 1)
        {
            inword >>= 1;		// shift right 1 bit
            inword &= 0b0000011111111111; // mask out only 11 lsbits
			
            return(inword); 
        }
		else return(0); // send zeros for bad data

}		// end read_mlx3

void init_lis(void)
{
write_lis_register(0x20, 0b10000111); //Power on device, enable all axis, turn off self test, CTRL_REG1
}	//end init_lis

int read_lis_x(void)
{
word_union x;

#define x16 x.i16
#define xhigh x.i8.byte1
#define xlow x.i8.byte0

		//xlow = read_lis_register(0x28); //Read outx_l
        //xhigh = read_lis_register(0x29); //Read outx_h
		xlow = read_lis_register(0x2A); //Read outx_l
        xhigh = read_lis_register(0x2B); //Read outx_h

		return(x16);
}

int read_lis_y(void)
{
word_union y;

#define y16 y.i16
#define yhigh y.i8.byte1
#define ylow y.i8.byte0

       	//ylow = read_lis_register(0x2A); //Read outy_l
        //yhigh = read_lis_register(0x2B); //Read outy_h
		ylow = read_lis_register(0x28); //Read outy_l
        yhigh = read_lis_register(0x29); //Read outy_h

		return(y16);
}

int read_lis_z(void)
{
word_union z;

#define z16 z.i16
#define zhigh z.i8.byte1
#define zlow z.i8.byte0

        zlow = read_lis_register(0x2C); //Read outz_l
        zhigh = read_lis_register(0x2D); //Read outz_h
        
		return(z16);
}

//Sends read command to LIS
char read_lis_register(char register_name)
{
    char in_byte;
	//unsigned int data;

    register_name |= 0b10000000; //Set D7 to 1 for read mode
    CS0=0; // = 0; //Select LIS
    WriteSPI(register_name); // write
	ReadSPI(); // dummy read to clear
	WriteSPI(0); // write to clock in bits
    in_byte = ReadSPI();
    CS0=1; //cs0 = 1;
    
    return(in_byte);
} 	//end read_lis_register

//Sends a write command to LIS
void write_lis_register(char register_name, char register_value)
{
    register_name &= 0b01111111; //Clear D7 to 0 for write mode
    CS0=0;  //Select LIS
    WriteSPI(register_name); //SPI write
	ReadSPI(); // dummy read to clear
    WriteSPI(register_value);
	ReadSPI(); // dummy read to clear
    CS0=1; //cs0 = 1;
    //Return nothing
}	// end write_lis_register

unsigned int ReadSPI()
{         
    /* Check for Receive buffer full status bit of status register*/
    //if (SPI2STATbits.SPIRBF) debugger messes this up
    { 
        SPI2STATbits.SPIROV = 0; // clear overflow bit
                
        //if (SPI2CONbits.MODE16)
        //    return (SPI2BUF);           /* return word read */
        //else
 
        return (SPI2BUF & 0xff);    /* return byte read */
    }
    //return -1;                 		/* RBF bit is not set return error*/
}

void WriteSPI(unsigned int data_out)
{   
    //if (SPI2CONbits.MODE16)          /* word write */
    //    SPI2BUF = data_out;
    //else 
    SPI2BUF = data_out & 0xff;   /*  byte write  */

	while (!IFS2bits.SPI2IF); 	//wait for transmit to complete
	IFS2bits.SPI2IF = 0;	//clear interrupt flag

}
