// servo functions

#include "microcontroller.h"
#include "servo.h"

#define servo_pin_mask 0b0000000000111111 // mask
#define servo_center FCY/100*152/1000	//1520*FCY/1000000   //count corresponding to 1520 us

//#define servo_center 0xAF18


int servo_mask;		// mask for received servo pulse
int prev_servo_mask;
int servo_chan_out;		// number for servo pulse being sent
unsigned int pulse_in[6]={servo_center,servo_center,servo_center,servo_center,servo_center,servo_center}; 	// measured input pulse for each servo input channel
unsigned int pulse_out[6]={servo_center,servo_center,servo_center,servo_center,servo_center,servo_center}; 	// pulse to output for each servo output channel



// initialize servo i/o
void init_servos(void)

	
{	
	//int i;
	prev_servo_mask = 0;	// initialize
	servo_chan_out = 0; 		// need 0 to start

	// set up output pulse pins
	PORTE=0;						// clear port
	TRISE =  0b0000000000000000;	// all outputs


	// set up input pulse pins
	ADPCFG = 0b1111111111111111; 	// analog inputs all off
	PORTB=0;  //clear all outputs
	TRISB =  0b0000000000111111; 	// RB0-5 are inputs - correspond to cn2-cn7
	CNEN1 =	 0b11111100;			// enable 5 CN pin interrupts
	//CNPU1 =  0b11111100;			// enable weak pullups if an unused pin is hi the servo routine will mess up
	IFS1bits.CNIF = 0;				// clear the CN interrupt flag
	IPC4bits.CNIP = 5 ;     // priority 5 ???
	IEC1bits.CNIE  = 1 ;     // enable the interrupt	
}

int servo_value(unsigned int count) 	// given measured servo count returns
{
return count;
}
										// scaled value 



void __attribute__((__interrupt__,__no_auto_psv__)) _CNInterrupt(void) 
// process cn interrupt servo pulse edge received
// assumes input pulses are in sequence on all channels (don't overlap each other)
	{ 	int servo_mask;		// mask for received servo pulse

		// received a change on one of the servo input pins cn2-6
		servo_mask = PORTB;	//get the port state 
		servo_mask &= servo_pin_mask; // mask the lower 6 CN bits

		if (servo_mask==0)

		{ // this is the falling edge of an input pulse, save value based on which pin went up on previous cycle
		  switch(prev_servo_mask)
	      {
	      						
	      case 1:  		// bit 0
						pulse_in[0] = TMR1-7; //save the pulse width

						// use channel 0 input as a trigger to start a set of output pulses

						if (servo_chan_out==0) 	// prevent interruption in unlikely event a previous set is in 
												// progress - this could diminish the frame rate if outputs get
												// long compared to inputs
						{
						// start timer 2 with interrupt to generate a set of servo output pulses
						T2CONbits.TON = 0;		// stop the timer
						TMR2=0;					// zero timer
						IFS0bits.T2IF = 0;		// clear TMR2 interrupt flag
						IEC0bits.T2IE  = 1;    	// enable TMR2 interrupt

						PR2 = pulse_out[0]; 	// set timer period register for channel 0
						PORTEbits.RE0 = 1;		// start chan 0 output pulse

						T2CONbits.TON = 1;		// start the timer
						} // end if

	                  	break;

	      case 2:  		// bit 1
						pulse_in[1] = TMR1-10; //save the pulse width
	                  	break;
	      				
	      case 4:  		// bit 2
						pulse_in[2] = TMR1-1; //save the pulse width
				       	break;

		  case 8:  		// bit 3
						pulse_in[3] = TMR1-15; //save the pulse width
	                  	break;

		  case 16:  	// bit 4
						pulse_in[4] = TMR1-8; //save the pulse width
	                  	break;

		  case 32:  	// bit 5
						pulse_in[5] = TMR1-12; //save the pulse width
	                  	break;
	                  
	      default:    //do nothing error
	                  break;					
         
         }     					// end switch(prev_servo_mask)

		}
	    else
		{ // this is the start of a input new pulse
			prev_servo_mask = servo_mask;  //save which pin for the next interrupt
			TMR1=0;  // start the timer for this input pulse
		} //end if

	IFS1bits.CNIF = 0;				// clear the CN interrupt flag
	}





void __attribute__((__interrupt__,__no_auto_psv__)) _T2Interrupt(void)  // Timer 2 interrupt marks end of the 
																		//current servo output pulse
	{
	PORTE=0;				// pin (all pins) low

	servo_chan_out += 1;	// increment channel counter

	T2CONbits.TON = 0;		// stop the timer
	TMR2=0;					// zero timer 


	switch(servo_chan_out)
	      {
	      						
	      case 1:  		// channel 1
						PR2 = pulse_out[1]; 	// set timer period register for channel 1
						PORTEbits.RE1 = 1;		// start chan 1 output pulse
	                  	break;

	      case 2:  		// channel 2
						PR2 = pulse_out[2]; 	// set timer period register for channel 2
						PORTEbits.RE2 = 1;		// start chan 2 output pulse
	                  	break;
	      				
	      case 3:  		// channel 3
						PR2 = pulse_out[3]; 	// set timer period register for channel 3
						PORTEbits.RE3 = 1;		// start chan 3 output pulse
	                  	break;

		  case 4:  		// channel 4
						PR2 = pulse_out[4]; 	// set timer period register for channel 4
						PORTEbits.RE4 = 1;		// start chan 4 output pulse
	                  	break;

		  case 5:  		// channel 5
						PR2 = pulse_out[5]; 	// set timer period register for channel 5
						PORTEbits.RE5 = 1;		// start chan 5 output pulse
	                  	break;

	                  
	      default:    IEC0bits.T2IE  = 0;    	// done with this set of output pulses disable TMR2 interrupt
	                  servo_chan_out =0;		// reset to allow a new set of outputs to be started
						break;					
         
         }     					// end switch(servo_chan_out)

		T2CONbits.TON = 1;		// start TMR2
		IFS0bits.T2IF = 0;		// clear TMR2 interrupt flag

	} // end _T2Interrupt

