// timer.c

#include "micro_frc.h"

static const float ticks_per_second_over8 = (float)FCY/8; // divide by 8 for timer4 prescaler 8:1

void init_timer1()
{
	T1CONbits.TCS = 0;   // Internal clock (FOSC/4)
	T1CONbits.TCKPS = 0; // No prescaler
	T1CONbits.TON = 1;
	//TMR1 = 0;            // Reset counter
}


void init_timer2() 	//2 and 3 are combined
					// set up now - dont start or enable interrupt
{	T2CON=0;              	// turn off for now
	T2CONbits.T32=0;		// not 32 bit
	T2CONbits.TCS = 0;   	// Internal clock (FOSC/4)
	T2CONbits.TCKPS = 0; 	// No prescaler
	IEC0bits.T2IE  = 0;		// disable the interrupt for now
	IPC1bits.T2IP = 4 ;     // priority 5 ???
}

void init_timer4() //4 and 5 are combined
{	T4CON=0;              // turn off for now
	TMR4=0;
	T4CONbits.T32=0;		// not 32 bit
	T4CONbits.TCS = 0;   // Internal clock (FOSC/2)
	T4CONbits.TCKPS = 1; // 8:1 prescaler to prevent overflow - rather than do 32 bit calcs
	T4CONbits.TON = 1;			// turn on
}

void reset_timer4() //4 and 5 are combined
{	
	T4CONbits.TON = 0;			// turn off
	TMR4=0;
	T4CONbits.TON = 1;			// turn on
}


int dt_ticks() 	// Returns the time in ticks since the last call 
				// of this function (or last reset)
{
	int ticks = TMR4;
	reset_timer4();	
	return ticks;
}

float dt_seconds() 	// Returns the time in seconds since the last call 
					// of this function (or last reset)
{
	float ticks = (float)TMR4;
	reset_timer4();	
	return ticks / ticks_per_second_over8;
}

